/*
 * pppParse.c - Log file statistics generator for pppd dial in servers.
 * Please read the LICENSE file included with this software.
 *
 * Copyright (c) 1999 Kyler Jones.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by Kyler Jones.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"

/*	These are the only global variables. 
*/

int total_access;
int total_sbyte;
int total_rbyte;
int total_time, on_usr_cnt;
char online_str[MAXPORTS][256], online_usr[MAXPORTS][24];
usrsum sums[MAXUSERS];
int allcheck = 0;


int	main(int argc, char *argv[])
{
	FILE *logfile;
	fpos_t fposa, fposb;
	int eof_chk=0, num_user=0, found=0, i, j;
	char entry[2500], user_name[MAXUSERS][25], new_user[25], dtime[25];
	char dpid[25], port[25], iface[25];

/*	MONTH and USER default to 'all' if nothing is given on the
	command line.
*/
	char MONTH[3] = "all", USER[25] = "all", LOG[100];
	int num_lines, report=0, argadd=0;
	usrall usr;
	char wait_char[4] = "|/-\\";

/*	Parse the command line for options (only 'help' and 'r' at the
	moment)
*/
	
	if(argc>1 && strstr(argv[1], "-"))
	{
	    if(!strcmp(argv[1],"--help") || !strcmp(argv[1], "-help") ||
	    !strcmp(argv[1], "-?"))
	    {
	      printf("\nUsage: pppParse [-r] FILE [USER | all] [MONTH | all]\n\n");
	      printf("Example calls:\n\n");
	      printf("\tpppParse /var/log/messages\n\n");
	      printf("Will run pppParse in interactive mode with USER all and MONTH all\n\n");
	      printf("\tpppParse -r /var/log/messages\n\n");
	      printf("Will output a report for USER all and MONTH all\n\n");
	      printf("\tpppParse /var/log/messages all June\n\n");
	      printf("Will run pppParse in interactive mode with USER all and MONTH June\n\n");
	      printf("\tpppParse /var/log/messages joe all\n\n");
	      printf("Will output a report for USER joe and MONTH all\n\n");
	      printf("All command line variables are case sensitive.\n\n");
	      exit(1);
	    }
	    if(!strcmp(argv[1], "-r"))
	    {		
		report = 1;
		argadd = 1;
	    }
	}	
	if((argc >= 2 && !argadd)  ||  (argc >= 3 && argadd))
		strcpy(LOG, argv[1+argadd]);

	if ((logfile = fopen(LOG, "rb")) == NULL)
	{
		printf("Can't open input file %s. (Permissions?)\n", LOG);	
		printf("Usage: pppParse FILE [USER | all] [MONTH | all]\n");
		printf("Try 'pppParse --help' for more information\n\n");
		exit(1);
	}

	if((argc >= 3 && !argadd)  ||  (argc >= 4 && argadd))
		strcpy(USER, argv[2+argadd]);
	

	if((argc >= 4 && !argadd)  ||  (argc >= 5 && argadd))
		strncpy(MONTH, argv[3+argadd], 3);

	if(!report  &&  strcmp("all", USER))
		strcpy(USER, "all");

/*	Check if "all" users was specified.  If not, call main
	parsing routine once and exit (if '-r' option was specified).
*/ 	

	if(strcmp(USER, "all"))
	{
		
	   if(report)
	   {	
		printf("\nUsage Statistics for user %s:\n\n\n", USER);
		printf("Date & Time\t    Minutes\tBytes Sent\tBytes Received\n\n");
		num_lines = user_parse(logfile, USER, MONTH, &usr);	
		for(i=1; i<= num_lines; i++)
		{
			printf("%s\n", usr.info[i]);
		}
		printf("\n\nTotal Times Logged In: %d\n", usr.access_count);
		printf("Total Online Time: %d Minutes\n", usr.online_count);
		printf("Total Bytes Sent: %d\n", usr.sbyte_count);
		printf("Total Bytes Received: %d\n\n", usr.rbyte_count);
		}
	    }
	else
	{
	if(report)
		printf("Finding All User names...\n\n");

/*	Save our position in the file so that we can come back
	and gther our statistics after find all users.
*/

	fgetpos(logfile, &fposa);

/*	Main Loop for checking all users.
*/
	
	if(!report)
	{
		num_user += 1;
		strcpy(user_name[0], "All Users\0");	
	}
	while(!eof_chk)
	{	
/*	Check if end of file reached (get_line returns non zero if
	reached).  Then file parsing is done.
*/

		if(get_line(logfile, entry))
			break;	

/*	Check for a user logging in with pppd - this could be tweaked
	depending on your syslog output.  See the README file to see
	what this is looking for.
*/

		if(strstr(entry, "pppd") != NULL && (strstr(entry,"logged in")
		!= NULL || (strstr(entry, "started by") != NULL && 
		strstr(entry, "LOGIN") == NULL)) && ((strstr(entry, MONTH)
		!= NULL) || (strstr(MONTH, "all") != NULL)))

		{

/*	Check if this user name has already been found.
*/

			if(strstr(entry, "started by") == NULL)
				get_name(entry, "user", 5, ' ', new_user); 	
			else
				get_name(entry, "by", 3, ',', new_user);
	
			for(i = 0; i < num_user; i++)
			{
				if(strcmp(user_name[i], new_user) == 0)
					found = 1;
			}	
			if(!found)
			{
				strcpy(user_name[num_user], new_user);
					if(report)
					printf("%s\n", user_name[num_user]);
				num_user += 1;
			}
			found = 0;

		}

	}

/*	Main loop for parsing users one at a time.  Rewinds the file
	to the beginning and calls the main file parsing routine for
	each user found.  Keep in mind that nothing is displayed here
	unless the '-r' option was used.  If not, all this loop does
	is collect global statistics for the interactive program.
*/
	if(report)
	{
	  for(i = 0; i < num_user; i++)
	  {
		fsetpos(logfile, &fposa);

		printf("\nUsage Statistics for user %s:\n\n\n", user_name[i]);
		printf("Date & Time\t    Minutes\tBytes Sent\tBytes Received\n\n");

		num_lines = user_parse(logfile, user_name[i],  MONTH, &usr);	
		for(j = 1; j <= num_lines; j++)
		{
			printf("%s\n", usr.info[j]);
		}
		printf("\n\nTotal Times Logged In: %d\n", usr.access_count);
		printf("Total Online Time: %d Minutes\n", usr.online_count);
		printf("Total Bytes Sent: %d\n", usr.sbyte_count);
		printf("Total Bytes Received: %d\n\n", usr.rbyte_count);
		
	  }

/*	Output Global Statistics (only if "all" users specified).
*/

	  printf("\n\nTotal Usage Statistics:\n");
	  printf("\nTotal Number of Users: %d\n", num_user);
	  printf("Total Number of Log-Ins: %d\n", total_access);
	  printf("Total Time Online (All Users): %d Minutes\n", total_time);
	  printf("Total Bytes Sent (By Users): %d\n", total_sbyte);
	  printf("Total Bytes Received (By Users): %d\n\n\n\n", total_rbyte);
        	
	 }

	}

	i = 0;


/*	Output users currently logged in, along with port/interface details.
*/

	if(on_usr_cnt && report)
	{
		printf("Currently Online:\n\n");
		printf("User\t   Date/Time\t\tInterface\tPort\t\tPID\n\n");
	
		while(i < on_usr_cnt)
		{
			strncpy(dtime, online_str[i], 15);
			dtime[15] = '\0';
			get_name(online_str[i], "Connect:", 9, ' ', iface);
			get_name(online_str[i], "<-->", 5, '\n', port);
			get_name(online_str[i], "pppd", 5, ']', dpid);
			
			printf("%s\t   %s\t%s\t\t%s\t%s\n\n\n", 
			online_usr[i], dtime, iface, port, dpid);
			i++;
		}
	}

/*	If '-r' option wasn't specified, we go to the interactive
	(curses based) version of pppParse.  See window.c comments
	for details on the following routines.
*/

	if(!report)
	{
	
	main_win(MONTH);	

	w_win('|');
	
	j = 0;
	

	  for(i = 0; i < num_user; i++)
	  {
		w_win(wait_char[j]);
	
		fsetpos(logfile, &fposa);
		user_parse(logfile, user_name[i],  MONTH, &usr);	
		sums[i].rbyte_count = usr.rbyte_count;
		sums[i].sbyte_count = usr.sbyte_count;
		sums[i].online_count = usr.online_count;
		sums[i].access_count = usr.access_count;
		j += 1;
		if(j==4)
			j = 0;
	  }
/*	The alcheck variable tells the user parsing routine not to 
	add the statistics from recurring calls to the user parsing
	routine (since the interactive version can parse a user
	over and over agin).
*/
	allcheck = 1;
	u_win(num_user, user_name, logfile, 0, MONTH, online_usr, online_str
			, on_usr_cnt);
	p_win(on_usr_cnt, online_usr, online_str, 0);
	i_win(num_user);
	u_win(num_user, user_name, logfile, 1, MONTH, online_usr, online_str
			, on_usr_cnt);
	clean_win();

	system("clear");
	}

	fclose(logfile);
	exit(0);		
}
